/* Mouse example with debug channel, for Teensy USB Development Board
 * http://www.pjrc.com/teensy/usb_mouse.html
 * Copyright (c) 2009 PJRC.COM, LLC
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "usb_mouse.h"

#define LED_CONFIG	(DDRD |= (1<<6))
#define LED_ON		(PORTD &= ~(1<<6))
#define LED_OFF		(PORTD |= (1<<6))
#define CPU_PRESCALE(n)	(CLKPR = 0x80, CLKPR = (n))

#define ANALOG_REF_EXTERNAL     0x00
#define ANALOG_REF_VCC          0x40
#define ANALOG_REF_2_56V        0xC0

#define ANALOG_0                0x00
#define ANALOG_1                0x01
#define ANALOG_2                0x02
#define ANALOG_3                0x03
#define ANALOG_4                0x04
#define ANALOG_5                0x05
#define ANALOG_6                0x06
#define ANALOG_7                0x07
#define ANALOG_1_TO_0_X10       0x09
#define ANALOG_1_TO_0_X200      0x0B
#define ANALOG_3_TO_2_X10       0x0D
#define ANALOG_3_TO_2_X200      0x0F
#define ANALOG_0_TO_1           0x10
#define ANALOG_2_TO_1           0x12
#define ANALOG_3_TO_1           0x13
#define ANALOG_4_TO_1           0x14
#define ANALOG_5_TO_1           0x15
#define ANALOG_6_TO_1           0x16
#define ANALOG_7_TO_1           0x17
#define ANALOG_0_TO_2           0x18
#define ANALOG_1_TO_2           0x19
#define ANALOG_3_TO_2           0x1B
#define ANALOG_4_TO_2           0x1C
#define ANALOG_5_TO_2           0x1D
#define ANALOG_1_1V             0x1E
#define ANALOG_0V               0x1F

void adc_config(char);
unsigned int adc_read(void);

void adc_config(char input)
{
    ADCSRA = (1<<ADEN) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);  // enable ADC
    ADMUX = ANALOG_REF_VCC | input;  // select Input and Reference
}

unsigned int adc_read(void)
{
    unsigned char lsb;

    ADCSRA |= (1<<ADSC);               // begin the conversion
    while (ADCSRA & (1<<ADSC)) ;   // wait for the conversion to complete
    lsb = ADCL;                              // read the LSB first
    return (ADCH << 8) | lsb;          // read the MSB and return 10 bit result
}


int8_t circle[];

int main(void)
{
	int8_t x, y, *p;

	// set for 16 MHz clock
	CPU_PRESCALE(0);
	LED_CONFIG;
	LED_OFF;

	// Initialize the USB, and then wait for the host to set configuration.
	// If the Teensy is powered without a PC connected to the USB port,
	// this will wait forever.
	usb_init();
	PORTD |= (1<<0);
	while (!usb_configured()) /* wait */ ;

	// Wait an extra second for the PC's operating system to load drivers
	// and do whatever it does to actually be ready for input
	_delay_ms(1000);

	while (1) {
		_delay_ms(20);
		LED_ON;  // turn the LED on while moving the mouse
		p = circle;
adc_config(7);
unsigned char u_x = adc_read()/4;
adc_config(6);
unsigned char u_y = adc_read()/4;
y=(127-u_y);
x=(u_x-127);
if(y<4 && y>-4) y=0;
else if (y<0) y=y+4;
else y=y-4;

if(x<2 && x>-2) x=0;
else if (x<0) x=x+2;
else x=x-2;

usb_mouse_buttons(!(PIND & (1<<0)),0,0);
			usb_mouse_move(2*x, 3*y, 0);
		LED_OFF;
		// This sequence creates a right click
		//usb_mouse_buttons(0, 0, 1);
		//_delay_ms(10);
		//usb_mouse_buttons(0, 0, 0);
	}
}


int8_t PROGMEM circle[] = {
16, -1,
15, -4,
14, -7,
13, -9,
11, -11,
9, -13,
7, -14,
4, -15,
1, -16,
-1, -16,
-4, -15,
-7, -14,
-9, -13,
-11, -11,
-13, -9,
-14, -7,
-15, -4,
-16, -1,
-16, 1,
-15, 4,
-14, 7,
-13, 9,
-11, 11,
-9, 13,
-7, 14,
-4, 15,
-1, 16,
1, 16,
4, 15,
7, 14,
9, 13,
11, 11,
13, 9,
14, 7,
15, 4,
16, 1
};
